function y=deriv_b(u);

% **************************************************
% SIMULATION DE LA STABILISATION D'UN SATELLITE PAR 
% GRADIENT DE GRAVITE ET AMORTISSEMENT PAR DES
% MAGNETOCOUPLEURS FONCTIONNANT AVEC DES COMMANDES
% BASEES SUR LA DERIVEE DU CHAMP MAGNETIQUE TERRESTRE
% **************************************************

% ------------------  DEBUT PROGRAMME  ---------------

global coeff gain

% ************************************************
% Le vecteur d'entre pour le traitement du systme
% diffrentiel est  12 composantes.
%
%	u(1)=p    rotation absolue satellite sur x
%	u(2)=q    ...........................sur y
%	u(3)=r    ...........................sur z
%
%	u(4)=q0   Premire composante du quaternion
%	u(5)=q1   Deuxime ........................
%	u(6)=q2   Troisime .......................
%	u(7)=q3   Quatrime .......................
% *************************************************

% Les donnes inertielles seront fournies plus loin
%	u(8)=Ir   Inertie roulis
%       u(9)=It   ....... tangage
%	u(10)=Il  ....... lacet

% Avec des donnes particulires
%      u(12)=temps
%
% ainsi que la vitesse angulaire orbitale w0
% 	u(11)=w0
% Dans le bloc diagramme nous multiplexerons ces entres
%       7+3+1=11
%
% ******************************************************

% Composantes de la rotation
p=u(1);q=u(2);r=u(3);

% normalisation du quaternion d'entre
v=u(4:7)/norm(u(4:7));

% Composantes du quaternion
q0=v(1);q1=v(2);q2=v(3);q3=v(4);

% Donnes inertielles et constantes orbitales
Ir=u(8);It=u(9);Il=u(10);

% Donnes particulires
w0=u(11);k=6*w0^2;

% Matrice de  passage  l'aide du quaternion v

Pmat=passage(v);

% Temps de la simulation

tps=u(12);

% Champ magnetique en axes XYZ (repre orbital)

BX=coeff*cos(w0*tps);
BY=0;
BZ=-2*coeff*sin(w0*tps);

% CHAMP MAGNETIQUE EN AXES SATELLITE

B=inv(Pmat)*[BX;BY;BZ];  % changement d'axes (en colonne)
Bx=B(1);By=B(2);Bz=B(3);

% ************* Calcul de la matrice associe  B **************

matB=[-Bx^2 0 Bx*Bz;0 -Bx^2-Bz^2 0;Bx*Bz By*Bz -Bx^2];

% Rotation relative du satellite par rapport au repre orbital local (colonne)
% exprime en axes satellite

rot=[p-w0*Pmat(2,1);q-w0*Pmat(2,2);r-w0*Pmat(2,3)];

% ********  Moment magntique command en loi de drive de B **************
    
% Drive de B dans le repre orbital SXYZ _____ exprime sur XYZ (colonne)
        dBX=-w0*coeff*sin(w0*tps);
        dBY=0;
        dBZ=-2*w0*coeff*cos(w0*tps);
        dBXYZ=[dBX;dBY;dBZ];
  
    % Drive de B en axes satellites exprime en axes satellite (colonne)
    % compte tenu de la formule de drivation dans 2 repres
        dBxyz_S=inv(Pmat)*dBXYZ-cross(rot,[Bx;By;Bz]);

    % Couple exerc sur le satellite -KdB/dt^B   (colonne en axes satellite)  dBxyz_S
        couple_magn=-gain*cross(dBxyz_S,B);

% Autre faon de calculer le couple sur le satellite, en utilisant la matrice associe  B    
        couple_magn=gain*(matB*rot+cross(B,inv(Pmat)*dBXYZ));

% ***************************************************************************

% Calcul des drives de p q r avec le gradient de gravit seul

y(1)=(Il-It)/Ir*(-q*r+k*(q2*q3+q0*q1)*(2*q0^2+2*q3^2-1))+couple_magn(1)/Ir;
y(2)=(Ir-Il)/It*(-p*r+k*(q1*q3-q0*q2)*(2*q0^2+2*q3^2-1))+couple_magn(2)/It;
y(3)=(It-Ir)/Il*(-q*p+2*k*(q2*q3+q0*q1)*(q1*q3-q0*q2))+couple_magn(3)/Il;


% Calcul des drives du quaternion, avec les calculs intermdiaires

 p21=2*(q1*q2+q0*q3);
 p22=2*(q0^2+q2^2)-1;
 p23=2*(q2*q3-q0*q1);

% Matrice W* de rotation relative en tant que quaternion

Matrice_rotation=[0 -p+w0*p21 -q+w0*p22 -r+w0*p23
                 p-w0*p21 0 r-w0*p23 -q+w0*p22
		 q-w0*p22 -r+w0*p23 0 p-w0*p21
		 r-w0*p23 q-w0*p22 -p+w0*p21 0];

z=0.5*Matrice_rotation*[q0 q1 q2 q3]';

y=[y z']';

end
